#include "assert.h"

#include "stdio.h"
#include "string.h"
#include "stdlib.h"
#include "math.h"

#include "SDL.h"
#include "SDL_rotozoom.h"
#include "SDL_ttf.h"
#include "SDL_mixer.h"

#include "BList.h"

#include "auxiliar.h"

#include "samples.h"
#include "animscript.h"
#include "valuescript.h"
#include "conditionscript.h"
#include "actionscript.h"
#include "variables.h"
#include "mog2.h"
#include "object.h"
#include "activeobject.h"
#include "activeroom.h"
#include "activeMAP.h"
#include "activemog2.h"
#include "ActiveINTERFACE.h"

#include "samples.h"

#ifdef KITSCHY_DEBUG_MEMORY 
#include "debug_memorymanager.h"
#endif

extern int surface_flags;

void ActiveOBJECT::menu_draw(int sx,int sy,SDL_Surface *sfc,SDL_Rect *vp)
{
	/* Draw a Menu object: */ 

	if (menu_active==1 || menu_timmer>0) {
		List<SDL_Surface> items;
		char *i;
		List<char> il;
		int start_y=y;
		int height=0;
		int n=0;
		FONTCONTAINER *font=0;
		SDL_Color c;
		SDL_Surface *s,*s2;
		SDL_Rect r2;
		float global_zoom=1.0F;
		float global_transparency=1.0F;
		float appear_factor=0.0F;

		if (object->menu_appear_factor>0) {
			appear_factor=float(menu_timmer)/object->menu_appear_factor;
			if (menu_active==0) appear_factor=1.0F-appear_factor;
		} /* if */ 

		if ((object->menu_entry_effect_flags&0x01)!=0) {
			global_transparency=1.0F-appear_factor;			
		} /* if */ 

		if ((object->menu_entry_effect_flags&0x02)!=0) {
			global_zoom=1.0F-appear_factor;
		} /* if */ 

		if (text_font!=0) font=get_game()->active_game->get_font(text_font,text_size);

		if (menu_title!=0 && menu_title[0]!=0) {
			c.r=text_r;
			c.g=text_g;
			c.b=text_b;
			s=font->multiline_text_surface(menu_title,0,c,0);

			if (global_zoom!=1.0) {
				SDL_Surface *s2=rotozoomSurface(s, 0, global_zoom, SMOOTHING_ON);
				SDL_FreeSurface(s);
				s=s2;
			} /* if */ 

			items.Add(s);
			height+=s->h+2;
		} /* if */ 

		il.Instance(menu_items);
		il.Rewind();

		while(il.Iterate(i)) {
			int r=text_r,g=text_g,b=text_b;
			float zoom=1.0;

			if (i!=0 && i[0]!=0) {
				if (n==menu_selected) {
					/* DIFFERENT COLOR: */ 
					if ((object->menu_appearance_flags&0x02)!=0) {
						r=object->menu_text_r2;
						g=object->menu_text_g2;
						b=object->menu_text_b2;
					} /* if */ 

					/* ZOOM: */ 
					if ((object->menu_appearance_flags&0x04)!=0) {
						zoom=float(object->menu_zoom_factor)/100.0F;
					} /* if */ 

					/* BLINK: */ 
					if ((object->menu_appearance_flags&0x01)!=0 &&
						object->menu_blink_timmer>0) {
						float factor;

						factor=2*(float(menu_blink_timmer)/float(object->menu_blink_timmer));

						factor=float(0.5F+0.5F*sin(factor*M_PI));

	//					if (factor>1.0) factor=2.0F-factor;

						r=int((1-factor)*object->text_r+factor*object->menu_text_r2);
						g=int((1-factor)*object->text_g+factor*object->menu_text_g2);
						b=int((1-factor)*object->text_b+factor*object->menu_text_b2);
						zoom=(1-factor)*1.0F+factor*float(object->menu_zoom_factor)/100.0F;
					} /* if */ 
				} /* if */ 

				c.r=r;
				c.g=g;
				c.b=b;
				s=font->multiline_text_surface(i,0,c,0);

				zoom*=global_zoom;

				if (zoom!=1.0) {
					s2=rotozoomSurface(s, 0, zoom, SMOOTHING_ON);
					SDL_FreeSurface(s);
					s=s2;
				} /* if */ 

				items.Add(s);
				height+=s->h+2;
			} /* if */ 

			n++;
		} /* while */ 

		/* Draw the menu frame: */ 
		if ((object->menu_appearance_flags&0x08)!=0) {
			SDL_Surface *s=SDL_CreateRGBSurface(SDL_SWSURFACE,object->menu_dx,object->menu_dy,32,0,0,0,AMASK);
			SDL_FillRect(s,0,SDL_MapRGBA(s->format,object->text_frame_r1,object->text_frame_g1,object->text_frame_b1,object->text_frame_a1));
			r2.x=int(x);
			r2.y=int(y);
			SDL_BlitSurface(s,0,sfc,&r2);
			SDL_FreeSurface(s);
		} /* if */ 

		start_y=y+(object->menu_dy/2)-height/2;
		while(!items.EmptyP()) {
			s=items.ExtractIni();

			switch(text_h_align) {
			case 0:
				r2.x=int(x);
				break;
			case 2:
				r2.x=(x+object->menu_dx)-(s->w);
				break;
			default:
				r2.x=(x+object->menu_dx/2)-(s->w/2);
				break;
			} /* switch */ 
			r2.y=start_y;

			if (global_transparency<1.0F) {
				surface_fader(s,1.0,1.0,1.0,global_transparency,0);
			} /* if */ 

			SDL_BlitSurface(s,0,sfc,&r2);
			start_y+=s->h+2;

			SDL_FreeSurface(s);
		} /* while */ 

	} /* if */ 

} /* ActiveOBJECT::menu_draw */ 



bool ActiveOBJECT::menu_cycle(KEYBOARDSTATE **LVK,int n_LVKs)
{
		
	menu_blink_timmer++;
	if (menu_blink_timmer>=object->menu_blink_timmer) menu_blink_timmer=0;

	if (menu_timmer>0) menu_timmer--;

	return true;
} /* ActiveOBJECT::menu_cycle */ 
