#include "string.h"
#include "stdlib.h"

#include "SDL.h"
#include "SDL_ttf.h"

#include "BList.h"
#include "BWindow.h"

#include "auxiliar.h"

#include "BMultiListWnd.h"

#ifdef KITSCHY_DEBUG_MEMORY 
#include "debug_memorymanager.h"
#endif

BMultiListWnd::BMultiListWnd(int x,int y,int dx,int dy,List<char> *items,TTF_Font *font) : BWindow(x,y,dx,dy,font)
{
	int i;
	items_p=items;

	for(i=0;i<items_p->Length();i++) selected.Add(new bool(false));
} /* BMultiListWnd::BMultiListWnd */ 


BMultiListWnd::~BMultiListWnd()
{
	items_p=0;
} /* BListWnd::~BListWnd */ 


bool BMultiListWnd::cycle(int stx,int sty)
{
	BWindow::cycle(stx,sty);

	if (mouse_button&SDL_BUTTON(1) &&
		!(old_mouse_button&SDL_BUTTON(1)) &&
		local_mouse_x>=old_stx && 
		local_mouse_y>=old_sty &&
		local_mouse_x<old_endx &&
		local_mouse_y<old_endy) {
		List<char> l;
		char *f;
		List<bool> l2;
		bool *b;
		int y=2;
		int i=0;

		l.Instance(*items_p);
		l.Rewind();
		l2.Instance(selected);
		l2.Rewind();
		while(l.Iterate(f) && l2.Iterate(b)) {
			if (local_mouse_y>=y && 
				local_mouse_y<=y+TTF_FontHeight(font) &&
				local_mouse_x>=2 &&
				local_mouse_x<=get_text_width(f,font)+10) {	
				if (*b) *b=false;
				   else *b=true;
				set_dirty();
			} /* if */ 
			i++;
			y+=2+TTF_FontHeight(font);
		} /* while */ 
	} /* if */ 

	return true;
} /* BMultiListWnd::cycle */ 


void BMultiListWnd::contens_size(int *dx,int *dy)
{
	int max_y;

	*dx=old_dx;
	*dy=old_dy;

	max_y=2+items_p->Length()*(TTF_FontHeight(font)+2);

	if (max_y>*dy) *dy=max_y;
} /* BMultiListWnd::contens_size */ 


void BMultiListWnd::draw_contens(SDL_Surface *sfc,int stx,int sty,int dx,int dy)
{
	int y;
	char *tmp;
	List<char> l;
	List<bool> l2;
	bool *b;

	SDL_FillRect(sfc,0,0);

	BWindow::draw_contens(sfc,stx,sty,dx,dy);

	y=2;
	l.Instance(*items_p);
	l.Rewind();
	l2.Instance(selected);
	l2.Rewind();
	while(l.Iterate(tmp)  && l2.Iterate(b)) {
		if (*b) {
			SDL_Rect r;
			r.x=1-stx;
			r.y=(y-1)-sty;
			r.w=dx-2;
			r.h=TTF_FontHeight(font)+2;
			SDL_FillRect(sfc,&r,SDL_MapRGBA(sfc->format,255,0,0,0));
		} /* if */ 
		print_left(tmp,font,255,255,255,sfc,2-stx,y-sty);
		y+=2+TTF_FontHeight(font);
	} /* while */ 

} /* BMultiListWnd::draw_contens */ 


List<int> *BMultiListWnd::get_selected(void)
{
	List<int> *res;
	List<bool> l2;
	bool *b;
	int i;

	res=new List<int>;

	l2.Instance(selected);
	l2.Rewind();
	i=0;
	while(l2.Iterate(b)) {
		if (*b) res->Add(new int(i));
		i++;
	} /* while */ 

	return res;
} /* BMultiListWnd::get_selected */ 


List<char> *BMultiListWnd::get_selected_name(void)
{
	List<char> *res;
	char *tmp;
	List<char> l;
	char *c;
	List<bool> l2;
	bool *b;
	int i;

	res=new List<char>;

	l.Instance(*items_p);
	l.Rewind();
	l2.Instance(selected);
	l2.Rewind();
	i=0;
	while(l.Iterate(c) && l2.Iterate(b)) {
		if (*b) {
			tmp=new char[strlen(c)+1];
			strcpy(tmp,c);
			res->Add(tmp);
		} /* if */ 
		i++;
	} /* while */ 

	return res;
} /* BMultiListWnd::get_selected_name */ 


void BMultiListWnd::set_selected(int s)
{
	*selected[s]=true;
} /* BMultiListWnd::set_selected */ 


void BMultiListWnd::set_selected_name(char *name)
{
	List<char> l;
	char *c;
	int i=0;

	l.Instance(*items_p);
	l.Rewind();
	while(l.Iterate(c)) {
		if (strcmp(c,name)==0) *selected[i]=true;
		i++;
	} /* while */ 
} /* BMultiListWnd::set_selected_name */ 



void BMultiListWnd::set_unselected(int s)
{
	*selected[s]=false;
} /* BMultiListWnd::set_unselected */ 


void BMultiListWnd::set_unselected_name(char *name)
{
	List<char> l;
	char *c;
	int i=0;

	l.Instance(*items_p);
	l.Rewind();
	while(l.Iterate(c)) {
		if (strcmp(c,name)==0) *selected[i]=false;
		i++;
	} /* while */ 
} /* BMultiListWnd::set_unselected_name */ 


bool BMultiListWnd::selected_p(int i)
{
	if (i<selected.Length()) return *selected[i];
	return false;
} /* BMultiListWnd::selected_p */ 


bool BMultiListWnd::selected_p_name(char *name)
{
	List<char> l;
	char *c;
	int i=0;

	l.Instance(*items_p);
	l.Rewind();
	while(l.Iterate(c)) {
		if (strcmp(c,name)==0) return *selected[i];
		i++;
	} /* while */ 
	return false;
} /* BMultiListWnd::selected_p */ 


void BMultiListWnd::add_element_name(char *name)
{
	items_p->Add(name);
	selected.Add(new bool(false));
} /* BMultiListWnd::add_element */ 


void BMultiListWnd::sub_element_name(char *name)
{
	List<char> l;
	List<bool> l2;
	bool *b,found;
	char *c;

	l.Instance(*items_p);
	l.Rewind();
	l2.Instance(selected);
	l2.Rewind();
	found=false;
	while(!found && l.Iterate(c) && l2.Iterate(b)) {
		if (strcmp(c,name)==0) found=true;
	} /* while */ 
	if (found) {
		items_p->DeleteElement(c);
		delete c;
		selected.DeleteElement(b);
		delete b;
	} /* if */ 
} /* BMultiListWnd::sub_element */ 


void BMultiListWnd::sub_element(int sel)
{
	List<char> l;
	List<bool> l2;
	bool *b,found;
	char *c;
	int i=0;

	l.Instance(*items_p);
	l.Rewind();
	l2.Instance(selected);
	l2.Rewind();
	found=false;
	i=0;
	while(!found && l.Iterate(c) && l2.Iterate(b)) {
		if (i==sel) found=true;
		i++;
	} /* while */ 
	if (found) {
		items_p->DeleteElement(c);
		delete []c;
		selected.DeleteElement(b);
		delete b;
	} /* if */ 
} /* BMultiListWnd::sub_element */ 

