#include "string.h"
#include "stdlib.h"

#include "SDL.h"
#include "SDL_ttf.h"

#include "BList.h"
#include "auxiliar.h"

#include "variables.h"
#include "mog2.h"
#include "object.h"
#include "tiles.h"
#include "interfaces.h"

#include "BWindow.h"
#include "BActiveWindow.h"
#include "BStringEdDlg.h"
#include "BIntegerEdDlg.h"
#include "BChoiceDlg.h"

#include "IObjectWnd.h"

#include "editor.h"

#ifdef KITSCHY_DEBUG_MEMORY 
#include "debug_memorymanager.h"
#endif

IObjectWnd::IObjectWnd(int x,int y,int dx,int dy,EDITOR *e,TTF_Font *font) : BWindow(x,y,dx,dy,font) 
{
	BButton *b;

	editor=e;

	b=new BButton(0,0,120,12,"OBJECTS/BITMAPS",1);
	buttons.Add(b);

	b=new BButton(178,0,20,12,"+",2);
	buttons.Add(b);
	b=new BButton(134,0,20,12,"-",3);
	buttons.Add(b);

} /* IObjectWnd::IObjectWnd */ 


IObjectWnd::~IObjectWnd()
{
} /* IObjectWnd::~IObjectWnd */ 


bool IObjectWnd::cycle(int stx,int sty)
{
	BWindow::cycle(stx,sty);

	if (((mouse_button&SDL_BUTTON(3) &&
		 !(old_mouse_button&SDL_BUTTON(3))) ||
		 (mouse_button&SDL_BUTTON(1) &&
		 !(old_mouse_button&SDL_BUTTON(1)))) &&
		local_mouse_x>=old_stx && 
		local_mouse_y>=old_sty &&
		local_mouse_x<old_endx &&
		local_mouse_y<old_endy) {

		if (editor->iv_displaying_objects) {
			/* DISPLAYING OBJECTS: */ 
			List<OBJECT> l,todelete;
			OBJECT *o;
			int y=TTF_FontHeight(font)+25;

			l.Instance(editor->game->objects);
			l.Rewind();
			while(l.Iterate(o)) {
				if (local_mouse_y>=y && 
					local_mouse_y<=y+TTF_FontHeight(font) &&
					local_mouse_x>=0 &&
					local_mouse_x<=get_text_width(o->name,font)) {

					if (mouse_button&SDL_BUTTON(1) &&
						!(old_mouse_button&SDL_BUTTON(1))) {
						/* LEFT BUTTON: */ 
						editor->iv_selected_obj=o;
						set_dirty();
					} /* if */ 

					if (mouse_button&SDL_BUTTON(3) &&
						!(old_mouse_button&SDL_BUTTON(3))) {
						/* RIGHT BUTTON: */ 
						BStringEdDlg *dlg;
						BChoiceDlg *dlg2;
						char tmp[80];
						int res=0;
						int y,x;

						new_window_coordinates(200,50,&x,&y);

						dlg2=new BChoiceDlg(x,y,200,50,"Choose action:","Edit name","Delete",&res,font);
						dlg2->Execute();
						delete dlg2;

						if (res==1) {
							strcpy(tmp,o->name);
							new_window_coordinates(320,60,&x,&y);
							dlg=new BStringEdDlg(x,y,220,60,"Enter object name:",tmp,40," ",font);
							dlg->Execute();
							delete dlg;

							delete o->name;
							o->name=new char[strlen(tmp)+1];
							strcpy(o->name,tmp);
							set_dirty();
						} /* if */ 
						if (res==2) {
							if (editor->iv_selected_obj==o) editor->iv_selected_obj=0;
							todelete.Add(o);
							set_dirty();
						} /* if */ 
					} /* if */ 
				} /* if */ 

				y+=TTF_FontHeight(font)+2;
			} /* while */ 	

			while(!todelete.EmptyP()) {
				OBJECT *o;
				o=todelete.ExtractIni();
				editor->delete_object(o);
			} /* while */ 
		} else  {
			/* DISPLAYING BITMAPS: */ 
			List<TILE> l,todelete;
			TILE *t;
			int y;

			y=25+TTF_FontHeight(font);
			l.Instance(editor->game->tiles[editor->ov_bitmap_category]);
			l.Rewind();
			while(local_mouse_y>=y && l.Iterate(t)) {
				if (local_mouse_y>=y && local_mouse_y<y+t->r.h+4 &&
					local_mouse_x>=0 && local_mouse_x<t->r.w+4) {
					/* BITMAPS OPTIONS: */ 

					if (mouse_button&SDL_BUTTON(1) &&
						!(old_mouse_button&SDL_BUTTON(1))) {
						/* LEFT BUTTON: */ 
						editor->iv_selected_tile=t;
						set_dirty();
					} /* if */ 

					if (mouse_button&SDL_BUTTON(3) &&
						!(old_mouse_button&SDL_BUTTON(3))) {

						BChoiceDlg *dlg;
						int res=0;
						int y,x;

						x=mouse_x-110;
						y=mouse_y;
						if (y>(old_y+old_dy-60)) y=old_y+old_dy-60;
						if (x>(old_x+old_dx-220)) x=old_x+old_dx-220;
						if (x<0) x=0;

						dlg=new BChoiceDlg(x,y,200,50,"Choose action:","Delete",&res,font);
						dlg->Execute();
						delete dlg;

						if (res==1) {
							todelete.Add(t);
							set_dirty();
						} /* if */ 
					} /* if */ 
				} /* if */ 
					
				y+=t->r.h+6;
			} /* while */ 

			while(!todelete.EmptyP()) {
				TILE *t;
				t=todelete.ExtractIni();
				editor->delete_tile(t);
			} /* while */ 

		} /* if */ 
	} /* if */ 

	return true;
} /* IObjectWnd::cycle */ 


void IObjectWnd::contens_size(int *dx,int *dy)
{
	int max_x=0;
	int max_y=0;

	BWindow::contens_size(dx,dy);

	if (editor->iv_displaying_objects) {
		/* DISPLAYING OBJECTS: */ 
		List<OBJECT> l;
		OBJECT *o;
		
		
		max_x=0;
		max_y=TTF_FontHeight(font)+25+editor->game->objects.Length()*(TTF_FontHeight(font)+2);

		l.Instance(editor->game->objects);
		l.Rewind();
		while(l.Iterate(o)) {
			if (get_text_width(o->name,font)>max_x) max_x=get_text_width(o->name,font);
		} /* while */ 
	} else {
		/* DISPLAYING BITMAPS: */ 
		List<TILE> l;
		TILE *t;
		int y/*,j*/;
		
		max_x=get_text_width("BITMAPS:",font);

		y=25+TTF_FontHeight(font);
//		for(j=0;j<TILECATEGORIES;j++) {
			l.Instance(editor->game->tiles[editor->ov_bitmap_category]);
			l.Rewind();
			while(l.Iterate(t)) {
				if (t->r.w+4>max_x) max_x=t->r.w+4;
				y+=t->r.h+6;
			} /* while */ 
//		} /* for */ 
		max_y=y;
	} /* if */ 
	

	if (max_x>*dx) *dx=max_x;
	if (max_y>*dy) *dy=max_y;
} /* IObjectWnd::contens_size */ 


void IObjectWnd::draw_contens(SDL_Surface *sfc,int stx,int sty,int dx,int dy)
{
	int y;
	char tmp[16];

	SDL_FillRect(sfc,0,0);

	BWindow::draw_contens(sfc,stx,sty,dx,dy);

	sprintf(tmp,"%i",editor->ov_bitmap_category);
	print_left(tmp,font,255,255,255,sfc,160-stx,3-sty);

	if (editor->iv_displaying_objects) {
		/* DISPLAYING OBJECTS: */ 
		List<OBJECT> l;
		OBJECT *o;

		print_left("OBJECTS:",font,255,255,255,sfc,0-stx,17-sty);

		l.Instance(editor->game->objects);
		l.Rewind();
		y=TTF_FontHeight(font)+25;
		while(l.Iterate(o)) {
			if (editor->iv_selected_obj==o) {
				SDL_Rect r;
				r.x=0-stx;
				r.y=y-sty;
				r.w=get_text_width(o->name,font);
				r.h=TTF_FontHeight(font);
				SDL_FillRect(sfc,&r,0x00ff0000);
			} /* if */ 
			print_left(o->name,font,255,255,255,sfc,0-stx,y-sty);
			y+=TTF_FontHeight(font)+2;
		} /* while */ 
	} else {
		/* DISPLAYING BITMAPS: */ 
		List<TILE> l;
		TILE *t;

		print_left("BITMAPS:",font,255,255,255,sfc,0-stx,17-sty);

		y=25+TTF_FontHeight(font);
		l.Instance(editor->game->tiles[editor->ov_bitmap_category]);
		l.Rewind();
		while(l.Iterate(t)) {
			if (editor->iv_selected_tile==t) draw_rectangle(sfc,0-stx,(y-2)-sty,t->r.w+3,t->r.h+3,0x00ff0000);
										else draw_rectangle(sfc,0-stx,(y-2)-sty,t->r.w+3,t->r.h+3,0x00ffffff);
			t->draw(2-stx,y-sty,sfc);
			y+=t->r.h+6;
		} /* while */ 
	} /* if */ 
} /* IObjectWnd::draw_contens */ 


bool IObjectWnd::button_pressed(int ID)
{
	switch(ID) {
	case 1:
		/* Switch between Objects and Bitmaps: */ 
		{
			if (editor->iv_displaying_objects) editor->iv_displaying_objects=false;
										  else editor->iv_displaying_objects=true;
			set_dirty();
		}
		break;

	case 2:
		{
			editor->ov_bitmap_category++;
			if (editor->ov_bitmap_category>=TILECATEGORIES) editor->ov_bitmap_category=0;
			set_dirty();
		}
		break;

	case 3:
		{
			editor->ov_bitmap_category--;
			if (editor->ov_bitmap_category<0) editor->ov_bitmap_category=TILECATEGORIES-1;
			set_dirty();
		}
		break;

	default:
		break;
	} /* switch */ 

	return true;
} /* IObjectWnd::button_pressed */ 
